@@ -37,7 +37,7 @@ void printlistfunc(FILE *fout)
   for(i=0;lfunc[i].gpname;i++)
     fprintf(fout,"%s %s %s\n",lfunc[i].gpname,lfunc[i].proto.cname,(lfunc[i].proto.code?lfunc[i].proto.code:""));
 }
-void checkisvar(int aff, char *func, int *var, Gtype *t)
+void checkisvar(int aff, const char *func, int *var, Gtype *t)
 {
   *var=aff;
   switch(tree[*var].f)
@@ -56,7 +56,7 @@ void checkisvar(int aff, char *func, int *var, Gtype *t)
   }
 }
 
-void genequal(int aff, char *func, int *var, int *binf, Gtype *t)
+void genequal(int aff, const char *func, int *var, int *binf, Gtype *t)
 {
   if (tree[aff].f!=Faffect)
     die(aff,"incorrect syntax for %s",func);
@@ -186,30 +186,37 @@ void genblockfuncspec(int n, int p, gpfunc *gp)
   case GPinstall:
     {
       gpfunc *gp;
-      char *p;
+      const char *p;
       int d, nf;
+      /* We should not use genblock(y,n) since prototype is
+       * {rrD\"\",r,D\"\",s,}
+       * Instead we fix Fentry to Fconst.
+       */
       genblock(y,n);
       nb=genlistargs(n,arg,2,4);
       /*What to do with arg[3]? add a linking ??*/
       if (nb<3 || arg[2]==GNOARG) arg[2]=arg[0];
       d=detag(arg[2]);
-      if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+      if (tree[d].f!=Fentry && !is_const(d,CSTstr))
         die(n,"Incorrect GP name in install");
+      tree[d].f=Fconst;
       nf=getfunc(entryname(d)); gp=lfunc+nf;
       d=detag(arg[0]);
-      if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+      if (tree[d].f!=Fentry && !is_const(d,CSTstr))
         die(n,"Incorrect C name in install");
+      tree[d].f=Fconst;
       gp->proto.cname=entryname(d);
       if (nb==4 && arg[3]!=GNOARG)
       {
 	d=detag(arg[3]);
-	if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+	if (!is_const(d,CSTstr))
 	  die(n,"Incorrect library name in install");
 	gp->proto.origin=entryname(d);
       }
       d=detag(arg[1]);
-      if (tree[d].f!=Fentry && tree[d].f!=Fstring)
+      if (tree[d].f!=Fentry && !is_const(d,CSTstr))
         die(n,"Incorrect code in install");
+      tree[d].f=Fconst;
       p=entryname(d);
       gp->proto.code=p;
       if (gp->proto.code[0]=='v')
@@ -759,60 +766,61 @@ int gentypefuncspec(int n, gpfunc *gp)
       die(n,"Internal error : gentypefuncspec called with no spec");
   }
 }
-void genpercent(FILE *fout, int n)
+
+void genformat(FILE *fout, int cnb, int *carg)
 {
-  char *s=value[tree[n].x].val.str;
-  while(*s)
+  int arg[STACKSZ];
+  int i,j;
+  int larg[1];
+  gpdescarg *rule;
+  gpfunc *gp;
+  int ngp;
+  ngp=findfunction("_formatcode");
+  if (ngp<0) die(-1,"Cannot find description of _formatcode");
+  gp=lfunc+ngp;
+  for(j=0;j<cnb;j++)
   {
-    if (*s=='%')
-      fputc('%',fout);
-    fputc(*s++,fout);
-  }
-}
-void genformat(FILE *fout, int nb, int *arg)
-{
-  int i;
-  for(i=0;i<nb;i++)
-  {
-    switch(ctype[tree[arg[i]].t])
+    int nb=listcalltostack(carg[j],OPcat,arg,STACKSZ,"concatenation",carg[j]);
+    for(i=0;i<nb;i++)
     {
-      case Vbptr:
-      case Vsmall:
-	if (tree[arg[i]].f!=Fsmall)
-	  fprintf(fout,"%%ld");
-	else
-	  fprintf(fout,"%d",tree[arg[i]].x);
-	break;
-      case Vgen:
-	fprintf(fout,"%%Z");
-	break;
-      case Vstr:
-	if (tree[arg[i]].f!=Fstring)
-	  fprintf(fout,"%%s");
-	else
-	  genpercent(fout,arg[i]);
-	break;
-      case Vvoid:
-	break;
-      case Vulong:
-	fprintf(fout,"%%lu");
-	break;
-      case Vgptr:
-	die(-1,"Printing gptr doesn't make sense");
+      int n=arg[i];
+      if (n==GNOARG) continue;
+      larg[0]=n;
+      rule=descfindrules(1, larg, gp);
+      if (!rule) die(n,"No format for %s arg",Gname[tree[n].t]);
+      gencodedesc(fout,1,larg,rule,n);
     }
   }
 }
-void genformatarg(FILE *fout, int nb, int *arg)
+
+void genformatarg(FILE *fout, int cnb, int *carg)
 {
-  int i;
-  for(i=0;i<nb;i++)
-    if (tree[arg[i]].f!=Fstring && tree[arg[i]].f!=Fsmall)
+  int arg[STACKSZ];
+  int i,j;
+  int larg[1];
+  gpdescarg *rule;
+  gpfunc *gp;
+  int ngp;
+  ngp=findfunction("_formatcode");
+  if (ngp<0) die(-1,"Cannot find description of _formatcode");
+  gp=lfunc+ngp;
+  for(j=0;j<cnb;j++)
+  {
+    int nb=listcalltostack(carg[j],OPcat,arg,STACKSZ,"concatenation",carg[j]);
+    for(i=0;i<nb;i++)
     {
-      fprintf(fout,",");
-      if (ctype[tree[arg[i]].t]==Vbptr)
-	fprintf(fout,"*");
-      gencode(fout,arg[i]);
+      int n=arg[i];
+      if (n==GNOARG) continue;
+      larg[0]=n;
+      rule=descfindrules(1, larg, gp);
+      if (!rule) die(n,"Not format for %s arg",Gname[tree[n].t]);
+      if (rule->type!=Gvoid)
+      {
+        fprintf(fout,",");
+        gencast(fout,arg[i],rule->type);
+      }
     }
+  }
 } 
 
 void geninitfunc(FILE *fout)
@@ -835,9 +843,9 @@ void geninitfunc(FILE *fout)
 #endif
             genindent(fout);
             if (ctype[ctxstack[i].t]==Vgen)
-              fprintf(fout,"%s = polx[fetch_user_var(\"%s\")];\n",v->var,v->var);
+              fprintf(fout,"%s = polx[fetch_user_var(\"%s\")];\n",v->cvar,v->var);
             else if (ctxstack[i].t==Gvar)
-              fprintf(fout,"%s = fetch_user_var(\"%s\");\n",v->var,v->var);
+              fprintf(fout,"%s = fetch_user_var(\"%s\");\n",v->cvar,v->var);
 #if TEST_UNITIALISED
           }
           break;
@@ -870,30 +878,28 @@ void gencheckargs(FILE *fout)
   {
     ctxvar *v;
     int doelse=0;
+    gpdescarg *rule;
     larg[0]=detag(arg[i]);
     if (tree[larg[0]].f==Faffect)
       larg[0]=tree[larg[0]].x;
     larg[0]=detag(larg[0]);
     v=getvarinblock(larg[0], fc);
-    if (v->t!=Gnotype && ctype[v->t]==Vgen && v->initval>0)
+    if (ctype[v->t]==Vgen && v->initval>0)
     {
       doelse=1;
       genindent(fout);
-      fprintf(fout,"if ( !%s ) %s = ",v->var,v->var);
+      fprintf(fout,"if ( !%s ) %s = ",v->cvar,v->cvar);
       gencast(fout,v->initval,v->t);
       fprintf(fout,";\n");
     }
-    if (v->t!=Gnotype)
+    tree[larg[0]].t=v->t;
+    tree[larg[0]].comment=-1;
+    rule=descfindrules(1, larg, gp);
+    if (!rule || rule->args[0].v != v->t) continue;
+    genindent(fout);
+    if(doelse) fprintf(fout,"else ");
+    switch(rule->type)
     {
-      gpdescarg *rule;
-      tree[larg[0]].t=v->t;
-      tree[larg[0]].comment=-1;
-      rule=descfindrules(1, larg, gp);
-      if (!rule || rule->args[0].v != v->t) continue;
-      genindent(fout);
-      if(doelse) fprintf(fout,"else ");
-      switch(rule->type)
-      {
       case Gbool:
         fprintf(fout,"if (");
         gencodedesc(fout,1,larg,rule,nfunc);
@@ -911,9 +917,8 @@ void gencheckargs(FILE *fout)
       default:
         gencodedesc(fout,1,larg,rule,nfunc);
         break;
-      } 
-      fprintf(fout,";\n");
-    }
+    } 
+    fprintf(fout,";\n");
   }
 }
 
@@ -1245,7 +1250,7 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     break;
   case GPprint:
     nb=genlistargs(n,arg,0,STACKSZ);
-    fprintf(fout,"fprintferr(\"");
+    fprintf(fout,"pariputsf(\"");
     genformat(fout,nb,arg);
     if (!isfunc(n,"print1") && !isfunc(n,"printp1"))
       fprintf(fout,"\\n");
@@ -1303,7 +1308,7 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     for(i=0;i<bl->g.n;i++)
     {
       if (i) fprintf(fout,", ");
-      fprintf(fout,"&%s",ctxstack[bl->gcvar[i]].var);
+      fprintf(fout,"&%s",ctxstack[bl->gcvar[i]].cvar);
     }
     fprintf(fout," }");
     break;
@@ -1321,7 +1326,7 @@ void genentryspec(FILE *fout, int n, gpfunc *gp)
     }
     else if (bl->gc&(1<<GCupto))
     {
-      char *varname=ctxstack[bl->gcvar[0]].var;
+      const char *varname=ctxstack[bl->gcvar[0]].cvar;
       fprintf(fout,"%s = gerepileupto(",varname);
       gencode(fout,arg[0]);
       fprintf(fout,", %s);\n",varname);
